@@ -17,22 +17,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+"""
+Scheme.
+"""
+
+__all__ = ('Scheme', 'SchemeLily')
 
 import re
 
 
-from parce import *
-from parce.action import *
-from parce.rule import *
+from parce import Language, lexicon, skip, default_action, default_target
+from parce.action import (
+    Bracket, Character, Comment, Delimiter, Keyword, Name, Number, String)
+from parce.rule import TEXT, bygroup, ifmember, gselect
 
 RE_SCHEME_RIGHT_BOUND = r"(?=$|[)\s])"
-RE_SCHEME_FRACTION = r"-?\d+/\d+" + RE_SCHEME_RIGHT_BOUND
-RE_SCHEME_FLOAT = r"-?((\d+(\.\d*)|\.\d+)(E\d+)?)" + RE_SCHEME_RIGHT_BOUND
-RE_SCHEME_NUMBER = (
-    r"("
-    r"-?\d+|"
-    r"#(b[0-1]+|o[0-7]+|x[0-9a-fA-F]+)|"
-    r"[-+]inf.0|[-+]?nan.0"
+RE_SCHEME_NUMBER = (r"(#[eEiI])?(#d)?("             # #e, #i and/or #d prefix
+    r"([-+]?(?:(?:\d+(?:\.\d*)|\.\d+)(?:[eE]\d+)?))"# float
+    r"|[-+]?\d+(?:(/\d+)|())"                       # fraction, int
+    r"|#(?:([bB][-+]?[0-1]+)"                       # binary
+        r"|([oO][-+]?[0-7]+)"                       # octal
+        r"|([xX][-+]?[0-9a-fA-F]+))"                # hexadecimal
+    r"|[-+](?:([iI][nN][fF])|([nN][aA][nN]))\.0"    # inf, NaN
     r")" + RE_SCHEME_RIGHT_BOUND
 )
 
@@ -50,21 +56,24 @@ class Scheme(Language):
         yield r'"', String, pop, cls.string
         yield r';', Comment, pop, cls.singleline_comment
         yield r'#!', Comment, pop, cls.multiline_comment
-        yield RE_SCHEME_FRACTION, Number.Fraction, pop
-        yield RE_SCHEME_FLOAT, Number.Float, pop
-        yield RE_SCHEME_NUMBER, Number, pop
+        yield RE_SCHEME_NUMBER, bygroup(
+            Number.Prefix, Number.Prefix,
+            gselect(None, None, None,
+                Number.Float, Number.Fraction, Number.Int,
+                Number.Binary, Number.Octal, Number.Hexadecimal,
+                Number.Infinity, Number.NaN)), pop
         if pop == 0:
             yield r"\.(?!\S)", Delimiter.Dot
-        yield r"#[tf]\b", Boolean, pop
+        yield r"#[tTfF]\b", Number.Boolean, pop
         yield r"#\\([a-z]+|.)", Character, pop
         yield r'[^()"{}\s]+', cls.get_word_action(), pop
 
-    @lexicon
+    @lexicon(consume=True)
     def list(cls):
         yield r"\)", Delimiter.CloseParen, -1
         yield from cls.common()
 
-    @lexicon
+    @lexicon(consume=True)
     def vector(cls):
         yield r"\)", Delimiter.CloseVector, -1
         yield from cls.common()
@@ -76,7 +85,7 @@ class Scheme(Language):
         return ifmember(TEXT, scheme_words.keywords, Keyword, Name)
 
     # -------------- String ---------------------
-    @lexicon
+    @lexicon(consume=True)
     def string(cls):
         yield r'"', String, -1
         yield from cls.string_common()
@@ -87,12 +96,12 @@ class Scheme(Language):
         yield default_action, String
 
     # -------------- Comment ---------------------
-    @lexicon
+    @lexicon(consume=True)
     def multiline_comment(cls):
         yield r'!#', Comment, -1
         yield from cls.comment_common()
 
-    @lexicon(re_flags=re.MULTILINE)
+    @lexicon(re_flags=re.MULTILINE, consume=True)
     def singleline_comment(cls):
         yield from cls.comment_common()
         yield r'$', Comment, -1
@@ -100,13 +109,18 @@ class Scheme(Language):
 
 class SchemeLily(Scheme):
     """Scheme used with LilyPond."""
-    @lexicon
-    def one_arg(cls):
+    @lexicon(consume=True)
+    def scheme(cls):
         """Pick one thing and pop back."""
         yield r'\s+', skip
-        yield from cls.common(-1)
+        yield from cls.common(cls.argument)
         yield default_target, -1
 
+    @lexicon(consume=True)
+    def argument(cls):
+        """One Scheme expression."""
+        yield default_target, -2
+
     @classmethod
     def common(cls, pop=0):
         from . import lilypond
