@@ -18,7 +18,7 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 
-"""
+r"""
 The Lexer is responsible for parsing text using Lexicons.
 
 The lexer generates Event objects, which contain a target (or None) and one or
@@ -56,7 +56,7 @@ Example::
     Event(target=Target(pop=-1, push=[]), tokens=((15, ';', Delimiter),))
     Event(target=Target(pop=-1, push=[]), tokens=((17, '}', Delimiter),))
 
-There is a convenience function in the ``parce`` module namespace that calls
+There is a convenience function in the *parce* module namespace that calls
 Lexer for you::
 
     >>> import parce
@@ -118,39 +118,53 @@ class Lexer:
         get_target = target_factory.get # access methods directly (faster)
         add_target = target_factory.add
         circular = set()
+
+        def event():
+            # yield Event, all vars are nonlocal :-)
+            if isinstance(action, ActionItem):
+                tokens = tuple(action.replace(self, pos, txt, match))
+                if tokens:
+                    yield Event(get_target(), tokens)
+            else:
+                yield Event(get_target(), ((pos, txt, action),))
+
         while True:
             for pos, txt, match, action, target in lexicons[-1].parse(text, pos):
-                if txt:
-                    if isinstance(action, ActionItem):
-                        tokens = tuple(action.replace(self, pos, txt, match))
-                        if tokens:
-                            yield Event(get_target(), tokens)
-                    else:
-                        yield Event(get_target(), ((pos, txt, action),))
                 if target:
-                    if target.pop:
-                        # never pop off root
-                        if -target.pop < len(lexicons):
-                            del lexicons[target.pop:]
-                        else:
-                            target = Target(1 - len(lexicons), target.push)
+                    # never pop off root
+                    if target.pop and -target.pop >= len(lexicons):
+                        target = Target(1 - len(lexicons), target.push)
+                    if txt:
+                        if target.push and target.push[-1].consume:
+                            # lexicon wants the starting tokens; handle target now
                             if target.pop:
                                 del lexicons[target.pop:]
-                    if target.push:
-                        if not txt:
-                            state = (pos, len(lexicons), len(target.push))
-                            if state in circular:
-                                if pos < len(text):
-                                    pos += 1
-                                circular.clear()
-                            else:
-                                circular.add(state)
+                            lexicons.extend(target.push)
+                            add_target(target)
+                            yield from event()
                         else:
+                            yield from event()
+                            if target.pop:
+                                del lexicons[target.pop:]
+                            lexicons.extend(target.push)
+                            add_target(target)
+                        circular.clear()
+                        pos += len(txt)
+                    else:
+                        if target.pop:
+                            del lexicons[target.pop:]
+                        state = (pos, len(lexicons), len(target.push))
+                        if state in circular:
+                            if pos < len(text):
+                                pos += 1
                             circular.clear()
+                        else:
+                            circular.add(state)
                         lexicons.extend(target.push)
-                    add_target(target)
-                    pos += len(txt)
+                        add_target(target)
                     break   # continue with new lexicon
+                elif txt:
+                    yield from event()
             else:
                 break   # done
 
