@@ -18,41 +18,48 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 
-"""
+r"""
 A Lexicon groups rules to match.
 
-A LexiconDescriptor is created by decorating a function yielding rules with the
-`@lexicon` decorator. When a LexiconDescriptor is accessed for the first time
-via a Language subclass, a Lexicon for that class is created and cached, and
-returned each time that attribute is accessed.
+A Lexicon is created by decorating a method yielding rules with the
+:attr:`@lexicon <lexicon>` decorator. (Although this actually
+creates a LexiconDescriptor. When a LexiconDescriptor is accessed for the first
+time via a Language subclass, a Lexicon for that class is created and cached,
+and returned each time that attribute is accessed.)
 
-The Lexicon can parse text according to the rules. When parsing for the first
-time, the rules-function is run with the language class as argument, and the
-rules it creates are cached.
+The Lexicon can parse text according to the rules. When its :func:`parse`
+function is called for the first time, the rules-function is run with the
+language class as argument, and the rules it yields are cached.
 
 This makes it possible to inherit from a Language class and only re-implement
 some lexicons, the others keep working as in the base class.
 
-Every Lexicon has the following attributes: (they are almost never needed, but
-anyway)
+Example:
 
-``arg``
-    The argument the lexicon was called with (creating a derived Lexicon).
-    None for a normal lexicon.
-``name``
-    The short name (name of the method it was defined with)
-``fullname``
-    The short name with the Language name prepended, like
-    ``'Language.lexicon'``.
-``qualname``
-    The full name with the Language's module prepended, like
-    ``'parce.lang.xml.Xml.root'``.
-``language``
-    The Language class the lexicon belongs to.
-``lexicon``
-    The LexiconDescriptor the Lexicon was created by.
-``re_flags``
-    The ``re_flags`` set by the @lexicon decorator.
+    >>> from parce import Language, lexicon
+    >>>
+    >>> class MyLang(Language):
+    ...     @lexicon
+    ...     def numbers(cls):
+    ...         yield r'\d+', "A number"
+    ...         yield r'\w+', "A word"
+    ...
+    >>> MyLang.numbers
+    MyLang.numbers
+    >>> type(MyLang.numbers)
+    <class 'parce.lexicon.Lexicon'>
+    >>> for i in MyLang.numbers.parse("1 a2 d3 4 p 5", 0):
+    ...  print(i)
+    ...
+    (0, '1', <re.Match object; span=(0, 1), match='1'>, 'A number', None)
+    (2, 'a2', <re.Match object; span=(2, 4), match='a2'>, 'A word', None)
+    (5, 'd3', <re.Match object; span=(5, 7), match='d3'>, 'A word', None)
+    (8, '4', <re.Match object; span=(8, 9), match='4'>, 'A number', None)
+    (10, 'p', <re.Match object; span=(10, 11), match='p'>, 'A word', None)
+    (12, '5', <re.Match object; span=(12, 13), match='5'>, 'A number', None)
+
+Parsing (better: lexing) is done by a :class:`~parce.lexer.Lexer` instance,
+which switches Lexicon when a target is encountered.
 
 """
 
@@ -69,10 +76,10 @@ from .ruleitem import (
 
 class LexiconDescriptor:
     """The LexiconDescriptor creates a Lexicon when called via a class."""
-    __slots__ = ('rules_func', 'lexicons', '_lock', 're_flags')
 
     def __init__(self, rules_func,
                        re_flags=0,
+                       consume=False,
         ):
         """Initializes with the rules function.
 
@@ -80,9 +87,10 @@ class LexiconDescriptor:
         the pattern, action, target, ... tuples.
 
         """
-        self.rules_func = rules_func
-        self.re_flags = re_flags
-        self.lexicons = {}
+        self.rules_func = rules_func    #: the function yielding the rules
+        self._re_flags = re_flags
+        self._consume = consume
+        self._lexicons = {}
         self._lock = threading.Lock()
 
     def __get__(self, instance, owner):
@@ -90,35 +98,56 @@ class LexiconDescriptor:
         if instance:
             raise RuntimeError('Language should never be instantiated')
         try:
-            return self.lexicons[owner]
+            return self._lexicons[owner]
         except KeyError:
             # prevent instantiating the same Lexicon multiple times
             with self._lock:
                 try:
-                    lexicon = self.lexicons[owner]
+                    lexicon = self._lexicons[owner]
                 except KeyError:
-                    lexicon = self.lexicons[owner] = Lexicon(self, owner)
+                    lexicon = self._lexicons[owner] = Lexicon(self, owner)
                 return lexicon
 
 
 class Lexicon:
-    """A Lexicon is tied to a particular class.
+    """A Lexicon parses text according to rules.
 
-    This makes it possible to inherit from a Language class and change
-    only some Lexicons.
+    A Lexicon is tied to a particular class, which makes it possible to inherit
+    from a Language class and change only some Lexicons.
 
-    Call Lexicon.parse(text, pos) to do the actual parsing work.
-    This function is created as soon as it is called for the first time.
+    .. py:function:: parse(text, pos)
+
+        Start parsing ``text`` from the specified position.
+        Yields five-tuples ``(pos, text, matchobj, action, target)``.
+
+        The ``pos`` is the start position a match was found, ``text`` is the
+        matched text, ``matchobj`` the match object (which can be None for
+        default actions), ``action`` the action that was specified in the
+        matching rule, and ``target`` is either None or a
+        :class:`~parce.target.Target` object.
 
     """
-    def __init__(self, lexicon, language, arg=None):
-        self.lexicon = lexicon
+    def __init__(self, descriptor, language, arg=None):
+        #: The LexiconDescriptor this Lexicon was created by.
+        self.descriptor = descriptor
+        #: The Language class the lexicon belongs to.
         self.language = language
+        #: The re_flags that were set on instantiation.
+        self.re_flags = descriptor._re_flags
+        #: Whether this lexicon wants the token(s) that switched to it
+        self.consume = descriptor._consume
+        #: The argument the lexicon was called with (creating a derived
+        #: Lexicon). None for a normal lexicon.
         self.arg = arg
-        self.name = lexicon.rules_func.__name__
+        #: The short name (name of the method this Lexicon was defined with)
+        self.name = descriptor.rules_func.__name__
+        #: The short name with the Language name prepended, like
+        #: ``'Language.lexicon'``.
         self.fullname = language.__name__ + '.' + self.name
+        #: The full name with the Language's module prepended, like
+        #: ``'parce.lang.xml.Xml.root'``.
         self.qualname = language.__module__ + '.' + self.fullname
-        self.__doc__ = lexicon.rules_func.__doc__
+        self.__doc__ = descriptor.rules_func.__doc__
         self._derived = {}
         # lock is used when creating a derivate and/or the parse() instance function
         self._lock = threading.Lock()
@@ -133,15 +162,24 @@ class Lexicon:
         parse time, which is useful for things like here documents, where you
         only get to know the end token after the start token has been found.
 
-        When comparing Lexicons with ``equals()``, a derivative lexicon
-        compares equal with the Lexicon that created them, although they
-        co-exist as separate objects.
+        When comparing Lexicons with ``equals()``, a derived lexicon compares
+        equal with the Lexicon that created them, although they co-exist as
+        separate objects.
+
+        When yielding the rules from a derived lexicon, the dynamic rule items
+        that depend on the Lexicon argument are already evaluated. When
+        yielding the rules from a vanilla lexicon, they are not evaluated, so
+        they adjust themselves to the lexicon they are included in (which will
+        then evaluate the rules of course).
 
         If arg is None, self is returned.
 
         """
-        if arg is None or self.arg is not None:
+        if arg is None:
             return self
+        elif self.arg is not None:
+            vanilla = self.descriptor.__get__(None, self.language)
+            return vanilla(arg)
         try:
             return self._derived[arg]
         except KeyError:
@@ -149,36 +187,45 @@ class Lexicon:
                 try:
                     lexicon = self._derived[arg]
                 except KeyError:
-                    lexicon = self._derived[arg] = Lexicon(self.lexicon, self.language, arg)
+                    lexicon = self._derived[arg] = Lexicon(self.descriptor, self.language, arg)
             return lexicon
 
     def equals(self, other):
         """Return True if we are the same lexicon or a derivate from the same."""
-        return self.lexicon is other.lexicon and self.language is other.language
+        return self.descriptor is other.descriptor and self.language is other.language
 
     @util.cached_property
     def _rules(self):
         """Yield the rules.
 
-        Patterns are created and rule items that depend on the lexicon argument
-        are evaluated.
+        Rule items that depend on the lexicon argument are not yet evaluated.
+
+        """
+        return tuple(self.descriptor.rules_func(self.language) or ())
+
+    @util.cached_property
+    def rules(self):
+        """Return all rules in a tuple.
+
+        Rule items that depend on the lexicon argument are already evaluated.
 
         """
         return tuple(pre_evaluate_rule(rule, self.arg)
-                for rule in self.lexicon.rules_func(self.language) or ())
+                for rule in self.descriptor.rules_func(self.language) or ())
 
     def __iter__(self):
         """Yield the rules.
 
-        Patterns are created and rule items that depend on the lexicon argument
-        are evaluated when this method is called for the first time.
+        Patterns are created when this method is called for the first time. If
+        this is a derived lexicon, dynamic rule items that depend on the
+        argument are already evaluated.
 
         """
-        yield from self._rules
+        yield from self.rules if self.arg else self._rules
 
     def __repr__(self):
         s = self.fullname
-        if self.arg is not None:
+        if self.arg:
             s += '*'
         return s
 
@@ -195,15 +242,10 @@ class Lexicon:
                 try:
                     return object.__getattribute__(self, name)
                 except AttributeError:
-                    self.parse = self.get_instance_attributes()
+                    self.parse = self._get_instance_attributes()
         return object.__getattribute__(self, name)
 
-    @property
-    def re_flags(self):
-        """The re_flags set on instantiation."""
-        return self.lexicon.re_flags
-
-    def get_instance_attributes(self):
+    def _get_instance_attributes(self):
         """Compile the pattern rules and return instance attributes.
 
         These are:
@@ -221,7 +263,7 @@ class Lexicon:
         make_target = TargetFactory.make
 
         # make lists of pattern, action and possible targets
-        for pattern, *rule in self:
+        for pattern, *rule in self.rules:
             if pattern is parce.default_action:
                 default_action = rule[0]
             elif pattern is parce.default_target:
@@ -231,7 +273,7 @@ class Lexicon:
                 patterns.append(pattern)
                 rules.append(rule)
 
-        # prepare to handle a dynamic default lexicon
+        # prepare to handle a dynamic default action
         if isinstance(default_action, RuleItem):
             def dynamic_default_action(text):
                 return default_action.evaluate({'text': text})
@@ -384,3 +426,37 @@ class Lexicon:
         return parse
 
 
+def lexicon(rules_func=None, **kwargs):
+    """Lexicon factory decorator.
+
+    Use this decorator to make a function in a Language class definition a
+    LexiconDescriptor object. The LexiconDescriptor is a descriptor, and when
+    calling it via the Language class attribute, a Lexicon is created, cached
+    and returned.
+
+    You can specify keyword arguments, that will be passed on to the Lexicon
+    object as soon as it is created.
+
+    The following keyword arguments are supported:
+
+    ``re_flags`` (0):
+        The flags that are passed to the regular expression compiler
+
+    ``consume`` (False):
+        When set to True, tokens originating from a rule that pushed this
+        lexicon are added to the target Context instead of the current.
+
+    The code body of the function should return (yield) the rules of the
+    lexicon, and is run with the Language class as first argument, as soon as
+    the lexicon is used for the first time.
+
+    You can also call the Lexicon object just as an ordinary classmethod, to
+    get the rules, e.g. for inclusion in a different lexicon.
+
+    """
+    if rules_func and not kwargs:
+        return LexiconDescriptor(rules_func)
+    def lexicon(rules_func):
+        return LexiconDescriptor(rules_func, **kwargs)
+    return lexicon
+
