@@ -209,6 +209,42 @@ def ifgroup(n, result, else_result=()):
     return select(call(operator.ne, MATCH(n), None), else_result, result)
 
 
+def gselect(*results, default=()):
+    """Yield one of the results if that group contributes to the match.
+
+    For example::
+
+        gselect(arg1, arg2, arg3, arg4, default=default)
+
+    is equivalent to::
+
+        ifgroup(1, arg1,
+            ifgroup(2, arg2,
+                ifgroup(3, arg3,
+                    ifgroup(4, arg4, default))))
+
+    When an ``arg`` is None, that group is skipped, so::
+
+        gselect(arg1, None, arg2, arg3)
+
+    is equivalent to::
+
+        ifgroup(1, arg1,
+            ifgroup(3, arg2,
+                ifgroup(4, arg3)))
+
+    """
+    indices, results = zip(*((i, r) for i, r in enumerate(results, 1) if r is not None))
+    indices = list(enumerate(indices))
+    default_index = len(results)
+    def predicate(m):
+        for i, n in indices:
+            if m.group(m.lastindex + n) is not None:
+                return i
+        return default_index
+    return select(call(predicate, MATCH), *results, default)
+
+
 def dselect(item, mapping, default=()):
     r"""Yield the ``item`` from the specified ``mapping`` (dictionary).
 
@@ -395,3 +431,42 @@ def using(lexicon):
     return ruleitem.DelegateAction(lexicon)
 
 
+### Helper to yield modified rules
+
+
+def anyof(lexicon, *target):
+    """Yield certain rules from the specified ``lexicon``, adding a ``target``.
+
+    Rules that specify a target themselves, and rules starting with
+    ``default_action`` or ``default_target``, are skipped. If no ``target`` is
+    specified, the ``lexicon`` becomes the target itself (specify ``0`` to
+    suppress that).
+
+    So when you use this function in a lexicon ``mylexicon`` like this::
+
+        @lexicon
+        def mylexicon(cls):
+            yield from anyof(cls.other_lexicon)
+
+        @lexicon
+        def other_lexicon(cls):
+            yield "patt1", Name.Symbol
+            yield "patt2", Delimiter, cls.yet_another_lexicon
+
+    the first rule of ``other_lexicon`` is yielded as::
+
+        ("patt1", Name.Symbol, cls.other_lexicon)
+
+    but the second rule ``"patt2"`` is not yielded, because it has a target
+    itself.
+
+    """
+    import parce.target
+    if not target:
+        target = lexicon,
+    for pattern, action, *t in lexicon:
+        if pattern not in (parce.default_action, parce.default_target) and \
+                not parce.target.TargetFactory.make(lexicon, t):
+            yield (pattern, action, *target)
+
+
