@@ -50,6 +50,7 @@ See also the documentation for Token and Context.
 
 import itertools
 import reprlib
+import weakref
 
 from parce import util
 from parce import query
@@ -70,11 +71,26 @@ DUMP_STYLE_DEFAULT = "round"
 
 class Node:
     """Methods that are shared by Token and Context."""
-    __slots__ = ()
+    __slots__ = ('__weakref__',)
 
     is_token = False
     is_context = False
 
+    @property
+    def parent(self):
+        """The parent Context (or None; uses a weak reference)."""
+        return self._parent()
+
+    @parent.setter
+    def parent(self, parent):
+        """Set the parent (to a Context or None)."""
+        self._parent = weakref.ref(parent) if parent is not None else lambda: None
+
+    @parent.deleter
+    def parent(self):
+        """Set the parent to None."""
+        self._parent = lambda: None
+
     def dump(self, file=None, style=None, depth=0):
         """Display a graphical representation of the node and its contents.
 
@@ -305,10 +321,10 @@ class Token(Node):
     `action`:
         the action specified by the lexicon rule that created the token
 
-    When a pattern rule in a lexicon matches the text, a Token is created.
-    When that rule would create more than one Token from a single regular
-    expression match, _GroupToken objects are created instead, carrying the
-    tuple of all instances in the `group` attribute. The `group` attribute is
+    When a pattern rule in a lexicon matches the text, a Token is created. When
+    that rule would create more than one Token from a single regular expression
+    match, _GroupToken objects are created instead, carrying the index of the
+    token in the group in the `group` attribute. The `group` attribute is
     readonly None for normal tokens.
 
     GroupTokens are thus always adjacent in the same context. If you want to
@@ -317,7 +333,9 @@ class Token(Node):
 
         t = ctx.find_token(45)
         if t.group:
-            t = t.group[0]
+            for t in t.left_siblings():
+                if not t.group:
+                    break
         pos = t.pos
 
     (A _GroupToken is just a normal Token otherwise, the reason a subclass was
@@ -358,7 +376,7 @@ class Token(Node):
 
     """
 
-    __slots__ = "parent", "pos", "text", "action"
+    __slots__ = "_parent", "pos", "text", "action"
 
     is_token = True
     group = None
@@ -400,6 +418,9 @@ class Token(Node):
         text = reprlib.repr(self.text)
         return "<Token {} at {}:{} ({})>".format(text, self.pos, self.end, self.action)
 
+    def __hash__(self):
+        return Node.__hash__(self)
+
     def __eq__(self, other):
         if isinstance(other, str):
             return other == self.text
@@ -479,8 +500,8 @@ class Token(Node):
 
         """
         node = self
-        if node.group:
-            node = node.group[-1]
+        if node.group is not None:
+            node = get_group_end(node)
         while node.parent:
             r = node.right_sibling()
             if r:
@@ -492,7 +513,15 @@ class Token(Node):
 
 class _GroupToken(Token):
     """A Token class that allows setting the `group` attribute."""
-    __slots__ = "group"
+    __slots__ = "group",
+
+    def __init__(self, group, parent, pos, text, action):
+        self.group = group
+        super().__init__(parent, pos, text, action)
+
+    def copy(self, parent=None):
+        """Return a copy of the Token, but with the specified parent."""
+        return type(self)(self.group, parent, self.pos, self.text, self.action)
 
 
 class Context(list, Node):
@@ -532,7 +561,7 @@ class Context(list, Node):
     might be in any sub-context of the current context.
 
     """
-    __slots__ = "lexicon", "parent"
+    __slots__ = "lexicon", "_parent"
 
     is_context = True
 
@@ -568,7 +597,7 @@ class Context(list, Node):
     def copy(self, parent=None):
         """Return a copy of the context, but with the specified parent."""
         # a non-recursive implementation due to Python's recursion limits
-        copy = type(self)(self.lexicon, parent)
+        copy = copy_root = type(self)(self.lexicon, parent)
         stack = []
         i = 0
         n = self
@@ -583,12 +612,15 @@ class Context(list, Node):
                     i = 0
                     n = m
                     break
-                elif m.group:
-                    group = tuple(node.copy(copy) for node in m.group)
-                    for m in group:
-                        m.group = group
-                    copy.extend(group)
-                    i += len(group)
+                elif m.group is not None:
+                    for g, j in enumerate(range(i + 1, z), m.group + 1):
+                        if n[j].is_context or n[j].group is None or n[j].group < g:
+                            copy.extend(node.copy(copy) for node in n[i:j])
+                            i = j
+                            break
+                    else:
+                        copy.append(m.copy(copy))
+                        i += 1
                 else:
                     copy.append(m.copy(copy))
                     i += 1
@@ -599,7 +631,7 @@ class Context(list, Node):
                     i = stack.pop() + 1
                 else:
                     break
-        return copy
+        return copy_root
 
     @property
     def pos(self):
@@ -647,9 +679,6 @@ class Context(list, Node):
                 else:
                     return height + 1
 
-        # originally, but that led to recursion errors...
-        #return max(n.height() + 1 if n.is_context else 1 for n in self) if self else 0
-
     def tokens(self):
         """Yield all Tokens, descending into nested Contexts."""
         stack = []
@@ -934,10 +963,13 @@ class Context(list, Node):
         is returned.
 
         """
+        prev = None
         for token in self.backward():
-            if token.group:
-                token = token.group[0]
-            return token
+            if not token.group:
+                return token
+            if prev and token.group >= prev.group:
+                return prev
+            prev = token
 
 
 def make_tokens(event, parent=None):
@@ -949,11 +981,40 @@ def make_tokens(event, parent=None):
 
     """
     if len(event.tokens) > 1:
-        tokens = tuple(_GroupToken(parent, *t) for t in event.tokens)
-        for t in tokens:
-            t.group = tokens
-        return tokens
+        return tuple(_GroupToken(n, parent, *t) for n, t in enumerate(event.tokens))
     else:
         return Token(parent, *event.tokens[0]),
 
 
+def get_group(token):
+    """For a token that belongs to a group, return the whole group as a list."""
+    p = token.parent
+    i = j = token.parent_index()
+    while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
+        i -= 1
+    z = len(p)
+    j += 1
+    while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
+        j += 1
+    return p[i:j]
+
+
+def get_group_start(token):
+    """For a token that belongs to a group, return the first token of the group."""
+    p = token.parent
+    i = token.parent_index()
+    while i and p[i].group > 0 and p[i-1].is_token and p[i-1].group is not None and p[i-1].group < p[i].group:
+        i -= 1
+    return p[i]
+
+
+def get_group_end(token):
+    """For a token that belongs to a group, return the last token of the group."""
+    p = token.parent
+    j = token.parent_index() + 1
+    z = len(p)
+    while j < z and p[j].is_token and p[j].group and p[j].group > p[j-1].group:
+        j += 1
+    return p[j-1]
+
+
