@@ -28,7 +28,7 @@ import collections
 import itertools
 
 from parce.lexer import Event, Lexer
-from parce.tree import Context
+from parce.tree import Context, get_group, get_group_start
 from parce.target import TargetFactory
 
 
@@ -38,9 +38,6 @@ BuildResult = collections.namedtuple("BuildResult", "tree start end offset lexic
 #: encapsulates the return values of :meth:`TreeBuilder.replace_tree`
 ReplaceResult = collections.namedtuple("ReplaceResult", "start end lexicons")
 
-#: encapsulates the return values of :func:`get_prepared_lexer`
-PreparedLexer = collections.namedtuple("PreparedLexer", "lexer events start")
-
 
 class Changes:
     """Store changes that have to be made to a tree.
@@ -131,16 +128,28 @@ def get_prepared_lexer(tree, text, start):
     """
     while start:
         last_token = start_token = find_token_before(tree, start)
-        while last_token and last_token.group and last_token.group[-1].end > start:
-            last_token = last_token.group[0].previous_token()
+        while last_token and last_token.group is not None:
+            group = get_group(last_token)
+            if group[-1].end <= start:
+                break
+            last_token = group[0].previous_token()
         if not last_token:
             return
         # go back at most 10 tokens, to the beginning of a group; if we
         # are at the first token just return 0.
         for start_token in itertools.islice(last_token.backward(), 10):
             pass
-        if start_token.group:
-            start_token = start_token.group[0]
+        while True:
+            if start_token.group:
+                start_token = get_group_start(start_token)
+            # go back further if this is the first token in a context whose
+            # lexicon has consume == True
+            if start_token.is_first() and start_token.parent.lexicon.consume:
+                start_token = start_token.previous_token()
+                if not start_token:
+                    return
+                continue
+            break
         start = start_token.pos if start_token.previous_token() else 0
         lexer = get_lexer(start_token) if start else Lexer([tree.lexicon])
         events = lexer.events(text, start)
@@ -191,9 +200,14 @@ def events_with_tokens(start_token, last_token):
                         break
                     else:
                         group = m,
-                        if m.group:
-                            count = len(m.group) - m.group.index(m)
-                            group = n[i:i+count]
+                        if m.group is not None:
+                            # find the last token in this group
+                            for g, j in enumerate(range(i + 1, z), m.group + 1):
+                                if n[j].is_context or n[j].group != g:
+                                    break
+                            else:
+                                j = z
+                            group = n[i:j]
                         tokens = tuple((t.pos, t.text, t.action) for t in group)
                         yield Event(get(), tokens), group
                         i += len(group)
@@ -217,14 +231,14 @@ def get_lexer(token):
 
 
 def new_tree(token):
-    """Return an empty context with the same ancestry as the token's."""
-    c = context = Context(token.parent.lexicon, None)
+    """Return an empty context (and its root) with the same ancestry as the token's."""
+    c = n = context = Context(token.parent.lexicon, None)
     for p in token.parent.ancestors():
         n = Context(p.lexicon, None)
         c.parent = n
         n.append(c)
         c = n
-    return context
+    return context, n
 
 
 def find_token_before(node, pos):
